﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 트랜잭션 데이터 요청
    /// </summary>
    /// <typeparam name="TItem">항목 타입</typeparam>
    public class TransactionDataRequest<TItem> : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 메시지 폼
        /// </summary>
        private IMessageForm messageForm;

        /// <summary>
        /// HTS
        /// </summary>
        protected AxKHOpenAPILib.AxKHOpenAPI hts;

        /// <summary>
        /// 요청명
        /// </summary>
        protected string requestName;

        /// <summary>
        /// 트랜잭션 코드
        /// </summary>
        protected string transactionCode;

        /// <summary>
        /// 화면 번호
        /// </summary>
        protected string screenNumber;

        /// <summary>
        /// 리소스 해제 여부
        /// </summary>
        protected bool disposed = false;

        /// <summary>
        /// 작업 여부
        /// </summary>
        protected bool isWorking = false;

        /// <summary>
        /// 입력값 딕셔너리
        /// </summary>
        protected Dictionary<string, string> inputValueDictionary = new Dictionary<string, string>();

        /// <summary>
        /// 결과 리스트
        /// </summary>
        protected List<TItem> resultList = new List<TItem>();

        /// <summary>
        /// 트랜잭션 데이터 타입
        /// </summary>
        protected TransactionDataType dataType;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 입력값 딕셔너리 - InputValueDictionary

        /// <summary>
        /// 입력값 딕셔너리
        /// </summary>
        public Dictionary<string, string> InputValueDictionary
        {
            get
            {
                return this.inputValueDictionary;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TransactionDataRequest(messageForm, hts, requestName, transactionCode, screenNumber, dataType)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="messageForm">메시지 폼</param>
        /// <param name="hts">HTS</param>
        /// <param name="requestName">요청명</param>
        /// <param name="transactionCode">트랜잭션 코드</param>
        /// <param name="screenNumber">화면 번호</param>
        /// <param name="dataType">트래잭션 데이터 타입</param>
        public TransactionDataRequest
        (
            IMessageForm               messageForm,
            AxKHOpenAPILib.AxKHOpenAPI hts,
            string                     requestName,
            string                     transactionCode,
            string                     screenNumber,
            TransactionDataType        dataType
        )
        {
            this.messageForm     = messageForm;
            this.hts             = hts;
            this.requestName     = requestName;
            this.transactionCode = transactionCode;
            this.screenNumber    = screenNumber;
            this.dataType        = dataType;

            this.hts.OnReceiveTrData += hts_OnReceiveTrData;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public
        //////////////////////////////////////////////////////////////////////////////// Function

        #region 리스트 구하기 - GetList()

        /// <summary>
        /// 리스트 구하기
        /// </summary>
        /// <returns>리스트</returns>
        public List<TItem> GetList()
        {
            try
            {
                this.isWorking = true;

                foreach(KeyValuePair<string, string> keyValuePair in this.inputValueDictionary)
                {
                    this.hts.SetInputValue(keyValuePair.Key, keyValuePair.Value);                
                }

                TimeSleepManager.Start();

                int returnCode = this.hts.CommRqData(this.requestName, this.transactionCode, 0, this.screenNumber);

                if(returnCode == 0)
                {
                    while(this.isWorking)
                    {
                        Thread.Sleep(300);

                        Application.DoEvents();
                    }

                    return this.resultList;
                }
                else
                {
                    return null;
                }
            }
            finally
            {
                this.isWorking = false;
            }
        }

        #endregion
        #region (IDisposable) 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            Dispose(true);

            GC.SuppressFinalize(this);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected
        //////////////////////////////////////////////////////////////////////////////// Function

        #region 항목 구하기 - GetItem(e, index)

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        /// <param name="index">인덱스</param>
        /// <returns>항목</returns>
        protected virtual TItem GetItem(AxKHOpenAPILib._DKHOpenAPIEvents_OnReceiveTrDataEvent e, int index)
        {
            return default(TItem);
        }

        #endregion
        #region 리소스 해제하기 - Dispose(disposing)

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        /// <param name="disposing">리소스 해제 여부</param>
        protected virtual void Dispose(bool disposing)
        {
            if(!this.disposed)
            {
                if(disposing)
                {
                    this.hts.OnReceiveTrData -= hts_OnReceiveTrData;

                    if(this.resultList != null)
                    {
                        this.resultList.Clear();

                        this.resultList = null;
                    }

                    this.disposed = true;
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region HTS 트랜잭션 데이터 수신시 처리하기 - hts_OnReceiveTrData(sender, e)

        /// <summary>
        /// HTS 트랜잭션 데이터 수신시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void hts_OnReceiveTrData(object sender, AxKHOpenAPILib._DKHOpenAPIEvents_OnReceiveTrDataEvent e)
        {
            if(e.sRQName != this.requestName && e.sTrCode != this.transactionCode && e.sScrNo != this.screenNumber)
            {
                return;
            }

            if(this.dataType == TransactionDataType.Single)
            {
                TItem item = GetItem(e, 0);

                this.resultList.Add(item);
            }
            else
            {
                int itemCount = this.hts.GetRepeatCnt(e.sTrCode, e.sRQName);

                for(int i = 0; i < itemCount; i++)
                {
                    TItem item = GetItem(e, i);

                    this.resultList.Add(item);
                }
            }

            if((this.dataType != TransactionDataType.Single) && (this.dataType != TransactionDataType.MultipleFirstPage) && (e.sPrevNext == "2"))
            {
                foreach(KeyValuePair<string, string> keyValuePair in this.inputValueDictionary)
                {
                    this.hts.SetInputValue(keyValuePair.Key, keyValuePair.Value);                
                }

                TimeSleepManager.Sleep();

                this.messageForm?.AddMessage($"{DateTime.Now.ToString("HH:mm:ss")} : 후속 데이터를 호출합니다.");

                int returnCode = this.hts.CommRqData(this.requestName, this.transactionCode, 2, this.screenNumber);
            
                if(returnCode != 0)
                {
                    this.resultList.Clear();
            
                    this.resultList = null;
            
                    this.isWorking = false;
                }
            }
            else
            {
              this.isWorking = false;
            }
        }

        #endregion
    }
}